@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2020 TileDB Inc.
+#  Copyright (c) 2017-2021 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -43,6 +43,7 @@ tiledb_array_schema.from_ptr <- function(ptr) {
 #' @param sparse (default FALSE)
 #' @param coords_filter_list (optional)
 #' @param offsets_filter_list (optional)
+#' @param capacity (optional)
 #' @param ctx tiledb_ctx object (optional)
 #' @examples
 #' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
@@ -64,6 +65,7 @@ tiledb_array_schema <- function(domain,
                                 sparse = FALSE,
                                 coords_filter_list = NULL,
                                 offsets_filter_list = NULL,
+                                capacity = 10000L,
                                 ctx = tiledb_get_context()) {
   if (!is(ctx, "tiledb_ctx")) {
     stop("ctx argument must be a tiledb_ctx")
@@ -104,6 +106,7 @@ tiledb_array_schema <- function(domain,
   }
   ptr <- libtiledb_array_schema(ctx@ptr, domain@ptr, attr_ptrs, cell_order, tile_order,
                                 coords_filter_list_ptr, offsets_filter_list_ptr, sparse)
+  libtiledb_array_schema_set_capacity(ptr, capacity)
   return(new("tiledb_array_schema", ptr = ptr))
 }
 
@@ -289,18 +292,56 @@ setMethod("tile_order", "tiledb_array_schema",
 #' @export
 setGeneric("filter_list", function(object, ...) standardGeneric("filter_list"))
 
+#' @rdname generics
+#' @param x A TileDB Object
+#' @export
+setGeneric("filter_list<-", function(x, value) standardGeneric("filter_list<-"))
+
 #' Returns the offsets and coordinate filter_lists associated with the `tiledb_array_schema`
 #'
 #' @param object tiledb_array_schema
 #' @return a list of tiledb_filter_list objects
 #' @export
-setMethod("filter_list", "tiledb_array_schema",
-          function(object) {
-            coords_ptr <- libtiledb_array_schema_get_coords_filter_list(object@ptr)
-            offsets_ptr <- libtiledb_array_schema_offsets_filter_list(object@ptr)
-            return(c(coords = tiledb_filter_list.from_ptr(coords_ptr),
-                     offsets = tiledb_filter_list.from_ptr(offsets_ptr)))
-          })
+setMethod("filter_list", "tiledb_array_schema", function(object) {
+  coords_ptr <- libtiledb_array_schema_get_coords_filter_list(object@ptr)
+  offsets_ptr <- libtiledb_array_schema_get_offsets_filter_list(object@ptr)
+  return(c(coords = tiledb_filter_list.from_ptr(coords_ptr),
+           offsets = tiledb_filter_list.from_ptr(offsets_ptr)))
+})
+
+# ' Set the Filter List for a TileDB Schema
+# '
+# ' @param x A TileDB Array Scheme
+# ' @param value A TileDB Filter List
+# ' @return The modified Array Schema object
+# ' @ export
+#setReplaceMethod("filter_list", "tiledb_array_schema", function(x, value) {
+#  x@ptr <- libtiledb_array_schema_set_coords_filter_list(x@ptr, value@ptr)
+#  x
+#})
+## -- need to provide setter for offsets and coords
+
+#' Set a Filter List for Coordinate of a TileDB Schema
+#'
+#' @param sch A TileDB Array Schema object
+#' @param fl A TileDB Filter List object
+#' @return The modified Array Schema object
+#' @export
+tiledb_array_schema_set_coords_filter_list <- function(sch, fl) {
+  sch@ptr <- libtiledb_array_schema_set_coords_filter_list(sch@ptr, fl@ptr)
+  sch
+}
+
+#' Set a Filter List for Variable-Sized Offsets of a TileDB Schema
+#'
+#' @param sch A TileDB Array Schema object
+#' @param fl A TileDB Filter List object
+#' @return The modified Array Schema object
+#' @export
+tiledb_array_schema_set_offsets_filter_list <- function(sch, fl) {
+  sch@ptr <- libtiledb_array_schema_set_offsets_filter_list(sch@ptr, fl@ptr)
+  sch
+}
 
 #' @rdname generics
 #' @export
@@ -364,27 +405,38 @@ dim.tiledb_array_schema <- function(x) dim(tiledb::domain(x))
 
 
 
-#' @rdname allows_dups-tiledb_array_schema-method
+#' @rdname tiledb_array_schema_get_allows_dups
 #' @export
 setGeneric("allows_dups", function(x) standardGeneric("allows_dups"))
 
+#' @rdname tiledb_array_schema_get_allows_dups
+#' @export
+setMethod("allows_dups", signature = "tiledb_array_schema", function(x) {
+  stopifnot(is.sparse(x))
+  libtiledb_array_schema_get_allows_dups(x@ptr)
+})
+
 #' Returns logical value whether the array schema allows duplicate values or not.
 #' This is only valid for sparse arrays.
 #'
 #' @param x tiledb_array_schema
 #' @return the logical value
 #' @export
-setMethod("allows_dups",
-          signature = "tiledb_array_schema",
-          function(x) {
-  stopifnot(is.sparse(x))
+tiledb_array_schema_get_allows_dups <- function(x) {
   libtiledb_array_schema_get_allows_dups(x@ptr)
-})
+}
 
-#' @rdname allows_dups-set-tiledb_array_schema-method
+#' @rdname tiledb_array_schema_set_allows_dups
 #' @export
 setGeneric("allows_dups<-", function(x, value) standardGeneric("allows_dups<-"))
 
+#' @rdname tiledb_array_schema_set_allows_dups
+#' @export
+setMethod("allows_dups<-", signature = "tiledb_array_schema", function(x, value) {
+  libtiledb_array_schema_set_allows_dups(x@ptr, value)
+  x
+})
+
 #' Sets toggle whether the array schema allows duplicate values or not.
 #' This is only valid for sparse arrays.
 #'
@@ -392,11 +444,9 @@ setGeneric("allows_dups<-", function(x, value) standardGeneric("allows_dups<-"))
 #' @param value logical value
 #' @return the tiledb_array_schema object
 #' @export
-setMethod("allows_dups<-", signature = "tiledb_array_schema",
-          function(x, value) {
+tiledb_array_schema_set_allows_dups <- function(x, value) {
   libtiledb_array_schema_set_allows_dups(x@ptr, value)
-  x
-})
+}
 
 ##' Get all Dimension and Attribute Names
 ##'
@@ -433,3 +483,89 @@ tiledb_schema_get_types <- function(sch) {
 
   alltypes <- c(dimtypes, attrtypes)
 }
+
+# -- get and set tile capacity
+
+#' @rdname tiledb_array_schema_get_capacity
+#' @export
+setGeneric("capacity", function(object) standardGeneric("capacity"))
+
+#' @rdname tiledb_array_schema_set_capacity
+#' @export
+setGeneric("capacity<-", function(x, value) standardGeneric("capacity<-"))
+
+#' @rdname tiledb_array_schema_get_capacity
+#' @export
+setMethod("capacity", signature = "tiledb_array_schema", function(object) {
+  libtiledb_array_schema_get_capacity(object@ptr)
+})
+
+#' @rdname tiledb_array_schema_set_capacity
+#' @export
+setReplaceMethod("capacity", signature = "tiledb_array_schema", function(x, value) {
+  libtiledb_array_schema_set_capacity(x@ptr, value)
+  x
+})
+
+#' Retrieve schema capacity (for sparse fragments)
+#'
+#' Returns the \code{tiledb_array} schema tile capacity for sparse fragments.
+#' @param object An \code{array_schema} object
+#' @return The tile capacity value
+#' @export
+tiledb_array_schema_get_capacity <- function(object) {
+  libtiledb_array_schema_get_capacity(object@ptr)
+}
+
+#' Sets the schema capacity (for sparse fragments)
+#'
+#' Sets the \code{tiledb_array} schema tile capacity for sparse fragments.
+#' @param x An \code{array_schema} object
+#' @param value An integer or numeric value for the new tile capacity
+#' @return The modified \code{array_schema} object
+#' @export
+tiledb_array_schema_set_capacity <- function(x, value) {
+  libtiledb_array_schema_set_capacity(x@ptr, value)
+  x
+}
+
+
+
+
+# -- Schema Correctness
+
+#' @rdname tiledb_array_schema_check
+#' @export
+setGeneric("check", function(object) standardGeneric("check"))
+
+#' @rdname tiledb_array_schema_check
+#' @export
+setMethod("check", signature = "tiledb_array_schema", function(object) {
+  libtiledb_array_schema_check(object@ptr)
+})
+
+#' Check the schema for correctness
+#'
+#' Returns the \code{tiledb_array} schema for correctness
+#' @param object An \code{array_schema} object
+#' @return The boolean value \code{TRUE} is returned for a correct
+#' schema; for an incorrect schema an error condition is triggered.
+#' @export
+tiledb_array_schema_check <- function(object) {
+  libtiledb_array_schema_check(object@ptr)
+}
+
+## -- convenience accessor `has_attribute` -- a little redundant as we already retrieve
+##    all names and can check the returned set but a direct caller is a little lighter
+
+#' Check a schema for a given attribute name
+#'
+#' @param schema A schema for a TileDB Array
+#' @param attr A character variable with an attribute name
+#' @return A boolean value indicating if the attribute exists in the schema
+#' @export
+has_attribute <- function(schema, attr) {
+  stopifnot(schema_argument=is(schema, "tiledb_array_schema"),
+            attr_argument=is.character(attr))
+  libtiledb_array_schema_has_attribute(schema@ptr, attr)
+}
