@@ -50,6 +50,7 @@ isNestedList <- function(l) {
   }
   return(FALSE)
 }
+
 ##' Look up TileDB type corresponding to the type of an R object
 ##'
 ##' Look up TileDB type corresponding to the type of an R object
@@ -71,56 +72,3 @@ r_to_tiledb_type <- function(x) {
     }
     type
 }
-
-##' Open a TileDB Array
-##'
-##' @param arr A TileDB Array object as for example returned by `tiledb_dense()`
-##' @param type A character value that must be either \sQuote{READ} or \sQuote{WRITE}
-##' @return The TileDB Array object but opened for reading or writing
-##' @export
-tiledb_array_open <- function(arr, type=c("READ","WRITE")) {
-  type <- match.arg(type)
-
-  arr@ptr <- libtiledb_array_open_with_ptr(arr@ptr, type)
-  arr
-}
-
-##' Close a TileDB Array
-##'
-##' @param arr A TileDB Array object as for example returned by `tiledb_dense()`
-##' @return The TileDB Array object but closed
-##' @export
-tiledb_array_close <- function(arr) {
-  libtiledb_array_close(arr@ptr)
-  arr
-}
-
-##' Check for Homogeneous Domain
-##'
-##' @param arr A TileDB Array object
-##' @return A boolean indicating if the array has homogeneous domains
-##' @export
-tiledb_array_is_homogeneous <- function(arr) {
-  ## there is a non-exported call at the C level we could use instead
-  sch <- schema(arr)
-  dom <- domain(sch)
-  domaintype <- sapply(libtiledb_domain_get_dimensions(dom@ptr),
-                       libtiledb_dim_get_datatype)
-  n <- length(unique(domaintype))
-  n == 1
-}
-
-##' Check for Heterogeneous Domain
-##'
-##' @param arr A TileDB Array object
-##' @return A boolean indicating if the array has heterogenous domains
-##' @export
-tiledb_array_is_heterogeneous <- function(arr) {
-  ## there is a non-exported call at the C level we could use instead
-  sch <- schema(arr)
-  dom <- domain(sch)
-  domaintype <- sapply(libtiledb_domain_get_dimensions(dom@ptr),
-                       libtiledb_dim_get_datatype)
-  n <- length(unique(domaintype))
-  n > 1
-}
